package cn.estsh.i3plus.pojo.base.jpa.dao;

import cn.estsh.i3plus.pojo.base.common.Pager;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.repository.NoRepositoryBean;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * @Description : 自定义Repository的方法接口
 * @Reference : 对jpaRepository进行扩展
 * @Author : alwaysfrin
 * @CreateDate : 2018-09-13 10:34
 * @Modify:
 **/
@NoRepositoryBean
public interface BaseRepository <T, ID extends Serializable> extends JpaRepository<T, ID> {

    /**
     * 新增对象<br/>
     * 注意：如果对象id是long，并且为0，则设为
     * 字符串，并且没有赋值，该方法将自动设置为uuid值
     * @param item
     *            持久对象，或者对象集合
     * @throws Exception
     */
    T insert(T item);

    /**
     * 更新对象数据
     *
     * @param item
     *            持久对象，或者对象集合
     * @throws Exception
     */
    void update(Object... item);

    /**
     * 执行ql语句
     * @param qlString 基于jpa标准的ql语句
     * @param values ql中的T参数值,单个参数值或者多个参数值
     * @return 返回执行后受影响的数据个数
     */
    int executeUpdate(String qlString, Object... values);

    /**
     * 执行ql语句
     * @param qlString 基于jpa标准的ql语句
     * @param params key表示ql中参数变量名，value表示该参数变量值
     * @return 返回执行后受影响的数据个数
     */
    int executeUpdate(String qlString, Map<String, Object> params);

    /**
     * 执行ql语句，可以是更新或者删除操作
     * @param qlString 基于jpa标准的ql语句
     * @param values ql中的T参数值
     * @return 返回执行后受影响的数据个数
     * @throws Exception
     */
    int executeUpdate(String qlString, List<Object> values);

    /**
     * 强删除
     */
    public void deleteById(ID id);
    public int deleteByProperty(String propName, Object propValue);
    public int deleteByPropertyIn(String propName, Object[] propValues);
    public int deleteByIds(ID[] ids);
    public int deleteByProperties(String[] propNames, Object[] objValues);

    /**
     * 通过多单属性进行单个值更新
     * @param conditionName
     * @param conditionValue
     * @param propertyName
     * @param propertyValue
     */
    public int updateByProperties(String conditionName, Object conditionValue, String propertyName, Object propertyValue);

    /**
     * 通过单个属性进行多个值更新
     * @param conditionName
     * @param conditionValue
     * @param propertyName
     * @param propertyValue
     */
    public int updateByProperties(String conditionName, Object conditionValue, String[] propertyName, Object[] propertyValue);

    /**
     * 通过多个属性进行单个值更新
     * @param conditionName
     * @param conditionValue
     * @param propertyName
     * @param propertyValue
     */
    public int updateByProperties(String[] conditionName, Object[] conditionValue, String propertyName, Object propertyValue);

    /**
     * 通过多个属性进行多个值更新
     * @param conditionName 查询属性名称
     * @param conditionValue 查询属性值
     * @param propertyName 更新属性名称
     * @param propertyValue 更新属性值
     */
    public int updateByProperties(String[] conditionName, Object[] conditionValue, String[] propertyName, Object[] propertyValue);

    /**
     * 根据 hqlWhere 批量修改制定字段
     * @param hqlWhere
     * @param propertyName  修改的字段名称
     * @param propertyValue 修改后的值
     * @return
     */
    public int updateByHqlWhere(String hqlWhere,String propertyName, Object propertyValue);

    /**
     * 批量修改数据熟悉
     * @param hqlWhere      HQL where 语句
     * @param propertyName  修改的字段名称
     * @param propertyValue 修改后的值
     * @return
     */
    public int updateByHqlWhere(String hqlWhere,String[] propertyName, Object[] propertyValue);

    public T getById(long id);

    public List<T> list();

    public List<T> list(final String hql);

    public List<T> listPager(Pager pager);

    public List<T> findByPage(final String hql, final int offset,final int pageSize);

    public List<T> findByPage(final String hql, final Object[] values, final int offset, final int pageSize);

    public List<T> findByProperty(String propertyName, Object value);

    public List<T> findByProperty(String[] propertyNames, Object[] values);

    public List<T> findByProperty(String propertyName, Object value,String orderByStuff);

    public List<T> findByProperty(String[] propertyNames, Object[] values,String orderByStuff);

    public T getByProperty(String propertyName, Object value);

    public T getByProperty(String[] propertyNames, Object[] values);

    public int listCount();

    public int findByPropertyCount(String propertyName, Object value);

    public int findByPropertyCount(String[] propertyNames, Object[] values);

    public List<T> findByPropertyPage(String propertyName, Object value,String orderByStuff,Pager pager);

    public List<T> findByPropertyPage(String[] propertyNames, Object[] values,String orderByStuff,Pager pager);

    public List<T> findByHql(String hql);

    public List<T> findByHqlWhere(String hqlWhere);

    public List<T> findByHqlWhere(String hqlWhere,String[] propertyNames, Object[] values,String orderByStuff);

    public int findByHqlWhereCount(String hqlWhere,String[] propertyNames, Object[] values);

    public List<T> findByHqlWherePage(String hqlWhere, Pager pager,String[] propertyNames, Object[] values,String orderByStuff);

    public List<T> findByHqlWherePage(String hqlWhere, Pager pager);

    public int findByHqlWhereCount(String hqlWhere);

    public int findByHqlCount(String hql);

    public int findByHqlCount(String hql,String[] paramName,Object[] paramValue);

    public List<T> findByHqlPage(String hql, Pager pager);

    public List<T> findByHqlPage(String hql, Pager pager,String[] paramName,Object[] paramValue);

    public List findBySql(String sql);

    public List<?> findBySql(String sql,Class entity);

    public List findBySqlPage(String sql,Pager pager);

    public List<?> findBySqlPage(String sql,Class entity,Pager pager);

    public int executeSql(String sql);

    public int findBySqlCount(String sql);

    public double findBySqlSumDouble(String sql,String columnName);

    public double findByHqlDouble(String hql);

    public long findByHqlLong(String hql);

    public List<T> findByHqlTopWhere(String hqlWhere,int topNum);

    public List<T> findColumnsByHql(String hql, String hqlWhere, String orderByStuff, Pager pager);

    /**********************  多表联查，返回Object[]数组根据hql查询翻页信息  开始 ****************************/
    public List<Object[]> findByHqlObjects(String hql);

    public List<Object[]> findByHqlObjects(String hql,String[] paramName,Object[] paramValue);

    public List<Object[]> findByHqlPageObjects(String hql, Pager pager);

    public List<Object[]> findByHqlPageObjects(String hql, Pager pager,String[] paramName,Object[] paramValue);

    public List<Object[]> findBySqlObjList(String sql);

    public List<Object[]> findBySqlObjListByPager(String sql,Pager pager);

    /****************** 部分业务处理 *******************/

    /**
     * 弱删除
     * @return count
     */
    public int deleteWeaklyById(ID id,String username);
    public int deleteWeaklyByIds(Long[] ids,String username) ;
    public int deleteWeaklyByProperty(String propName, Object propValue,String username);
    public int deleteWeaklyByPropertyIn(String propName, Object[] propValues,String username);
    public int deleteWeaklyByProperties(String[] propNames, Object[] objValues,String username);

    /**
     * 更新启用禁用
     * @return count
     */
    public int updateValidStatusById(ID id,int status,String username);
    public int updateValidStatusByIds(Long[] ids,int status,String username) ;
    public int updateValidStatusByProperty(String propName, Object propValue,int status,String username);
    public int updateValidStatusByPropertyIn(String propName, Object[] propValues,int status,String username);
    public int updateValidStatusByProperties(String[] propNames, Object[] objValues,int status,String username);

    public boolean isExitByProperty(String propertyName, Object value);

    public boolean isExitByProperty(String[] propertyNames, Object[] values);

    public boolean isExitByHqlWhere(String hqlWhere,String[] propertyNames, Object[] values);

    public boolean isExitByHqlWhere(String hqlWhere);

    public boolean isExitByHql(String hql);

    public boolean isExitByHql(String hql,String[] paramName,Object[] paramValue);

    public boolean isExitBySql(String sql);

    public double findSumByProperty(String sumPropertyName,String groupByName,String propertyName, Object value);

    public double findSumByProperties(String sumPropertyName,String groupByName,String[] paramName,Object[] paramValue);

    public double findAvgByProperty(String sumPropertyName,String groupByName,String propertyName, Object value);

    public double findAvgByProperties(String sumPropertyName,String groupByName,String[] paramName,Object[] paramValue);

    public double findMaxByProperty(String sumPropertyName,String groupByName,String propertyName, Object value);

    public double findMaxByProperties(String sumPropertyName,String groupByName,String[] paramName,Object[] paramValue);

    public double findMinByProperty(String sumPropertyName,String groupByName,String propertyName, Object value);

    public double findMinByProperties(String sumPropertyName,String groupByName,String[] paramName,Object[] paramValue);
}
